"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.previewApp = void 0;
const os_1 = __importDefault(require("os"));
const cals_1 = require("@cloudbase/cals");
const lowcode_builder_1 = require("@cloudbase/lowcode-builder");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const chokidar_1 = __importDefault(require("chokidar"));
const execa_1 = __importDefault(require("execa"));
const common_1 = require("../../common");
const glob_1 = require("glob");
const preview_1 = require("../preview");
const chalk_1 = __importDefault(require("chalk"));
const WEDA_PREVIEW_ROOT_PATH = path_1.default.resolve(os_1.default.homedir(), '.weda-preview');
const H5_PREVIEW_TMPL_ROOT_PATH = path_1.default.resolve(__dirname, '..', '..', 'template', 'h5_preview');
function getH5PreviewPath(appId) {
    return path_1.default.resolve(WEDA_PREVIEW_ROOT_PATH, appId, 'h5');
}
function getMPPreviewPath(appId) {
    return path_1.default.resolve(WEDA_PREVIEW_ROOT_PATH, appId, 'mp');
}
function globAsync(filePath) {
    return new Promise((resolve) => {
        (0, glob_1.glob)(filePath, (err, matches) => {
            resolve(matches);
        });
    });
}
async function buildPreviewH5App(data) {
    const tmplSrc = H5_PREVIEW_TMPL_ROOT_PATH;
    const destPath = data.outputPath;
    await fs_extra_1.default.copy(tmplSrc, destPath);
    await partialUpdatePreviewH5AppFiles(tmplSrc, destPath, data);
}
async function partialUpdatePreviewH5AppFiles(tmplDir, outputPath, data) {
    // update App.tsx
    const appTsxPath = path_1.default.resolve(tmplDir, 'src', 'App.tsx');
    const appTsxContent = await fs_extra_1.default.readFile(appTsxPath, 'utf-8');
    const replacedAppTsxContent = appTsxContent
        .replace('<%= appId %>', data.appId)
        .replace('<%= uin %>', data.uin)
        .replace('<%= envId %>', data.envId);
    await fs_extra_1.default.writeFile(path_1.default.resolve(outputPath, 'src', 'App.tsx'), replacedAppTsxContent, 'utf-8');
    // update dependencies
    await fs_extra_1.default.writeFile(path_1.default.resolve(outputPath, 'src', 'dependencies.ts'), `
  export default ${JSON.stringify(data.dependencies)}
  `, 'utf-8');
    // update cals
    await fs_extra_1.default.writeFile(path_1.default.resolve(outputPath, 'src', 'cals.ts'), `
  export default ${JSON.stringify(data.cals)} as any;
  export const sub_cals = ${JSON.stringify(data.sub_cals || [])} as any;
  `, 'utf-8');
}
async function previewApp(ctx, params) {
    var _a;
    const { projectPath } = ctx;
    const { platform = lowcode_builder_1.BuildType.WEB, wxDevtoolPath } = params;
    const data = await _generateBuildData();
    if (platform === lowcode_builder_1.BuildType.WEB) {
        const buildPreviewH5AppData = _getBuildPreviewH5AppData(data);
        const rootPath = getH5PreviewPath(data.appKey);
        common_1.Logger.info(`👉 构建预览应用中...`);
        await buildPreviewH5App(buildPreviewH5AppData);
        // Install dependencies
        common_1.Logger.info(`👉 安装依赖中...`);
        execa_1.default.commandSync('npm i --verbose', {
            cwd: rootPath,
            shell: true,
        });
        // start dev server
        common_1.Logger.info(`👉 启动开发服务器`);
        const subprocess = execa_1.default.command('npm run dev', {
            cwd: rootPath,
            shell: true,
        });
        (_a = subprocess.stdout) === null || _a === void 0 ? void 0 : _a.pipe(process.stdout);
    }
    else if (platform === lowcode_builder_1.BuildType.MP) {
        // First build, need to install independencies
        await (0, lowcode_builder_1.buildWedaApp)(data);
        const previewService = new preview_1.PreviewService({
            port: 0,
            mode: 'debug',
            wxDevtoolPath,
        });
        const rootPath = getMPPreviewPath(data.appKey);
        common_1.Logger.info('👉 开始构建小程序miniprogram_npm包...');
        await previewService.buildMiniprogramNpm(rootPath);
        common_1.Logger.info('👉 开始打开微信开发者工具...');
        await previewService.openIDE(rootPath);
        common_1.Logger.info(`👉 打开微信开发者工具成功。如没有正常打开，可直接手动打开这个目录：${chalk_1.default.bold.yellow(rootPath)}`);
        const { materialsDir } = (0, lowcode_builder_1.getCompileDirs)();
        common_1.Logger.info(`👉 如遇到问题，建议删除组件缓存重试下。删除目录：${chalk_1.default.bold.yellow(materialsDir)}`);
    }
    // Watch code changing in src dir
    chokidar_1.default
        .watch(path_1.default.resolve(projectPath, 'src'), {
        persistent: true,
        ignoreInitial: true,
    })
        .on('all', async (event, filePath) => {
        const data = await _generateBuildData();
        if (platform === lowcode_builder_1.BuildType.WEB) {
            const buildPreviewH5AppData = _getBuildPreviewH5AppData(data);
            const tmplSrc = H5_PREVIEW_TMPL_ROOT_PATH;
            const destPath = getH5PreviewPath(data.appKey);
            await partialUpdatePreviewH5AppFiles(tmplSrc, destPath, buildPreviewH5AppData);
        }
        else if (platform === lowcode_builder_1.BuildType.MP) {
            try {
                await (0, lowcode_builder_1.buildWedaApp)(Object.assign({}, data, { ignoreInstall: true }));
            }
            catch (e) {
                console.error(e);
            }
        }
    });
    async function _generateBuildData() {
        var _a, _b, _c, _d;
        const dependencies = await fs_extra_1.default.readJSON(path_1.default.resolve(projectPath, 'dependencies.json'), 'utf8');
        const codeList = await (0, common_1.fileToCode)(projectPath);
        const calsResult = (0, cals_1.codeToCals)(codeList, 'v0');
        const outputPath = platform === lowcode_builder_1.BuildType.MP ? getMPPreviewPath(calsResult.id) : getH5PreviewPath(calsResult.id);
        const subApps = await globAsync(path_1.default.resolve(projectPath, 'src', 'packages', '*'));
        let subAppCalsList = [];
        if (subApps.length > 0) {
            subAppCalsList = await Promise.all(subApps.map(async (subApp) => {
                const codeList = await (0, common_1.fileToCode)(subApp);
                return (0, cals_1.codeToCals)(codeList, 'v0');
            }));
        }
        const data = {
            cals: calsResult,
            subAppCalsList,
            dependencies,
            appKey: calsResult.id,
            runtime: lowcode_builder_1.RUNTIME.CLI,
            buildTypeList: [platform],
            mode: lowcode_builder_1.WebpackModeType.NONE,
            domain: ((_a = calsResult === null || calsResult === void 0 ? void 0 : calsResult.extra) === null || _a === void 0 ? void 0 : _a.domain) || '',
            output: { path: outputPath },
            // BuildType.MP 专用
            // @ts-ignore
            resourceAppId: (_b = calsResult === null || calsResult === void 0 ? void 0 : calsResult.extra) === null || _b === void 0 ? void 0 : _b.resourceAppId,
            deployOptions: {
                mode: lowcode_builder_1.DEPLOY_MODE.PREVIEW,
                // @ts-ignore
                mpAppId: (_c = calsResult === null || calsResult === void 0 ? void 0 : calsResult.extra) === null || _c === void 0 ? void 0 : _c.mpAppId,
                // targetMpAppId: 'wx5e6fc5d676b114d7',
            },
            // @ts-ignore
            endpointType: (_d = calsResult === null || calsResult === void 0 ? void 0 : calsResult.extra) === null || _d === void 0 ? void 0 : _d.endpointType,
        };
        return data;
    }
    function _getBuildPreviewH5AppData(data) {
        var _a, _b, _c, _d, _e, _f;
        return {
            outputPath: ((_a = data.output) === null || _a === void 0 ? void 0 : _a.path) || '',
            appId: ((_b = data.cals) === null || _b === void 0 ? void 0 : _b.id) || '',
            // @ts-ignore
            envId: ((_d = (_c = data.cals) === null || _c === void 0 ? void 0 : _c.extra) === null || _d === void 0 ? void 0 : _d.envId) || '',
            uin: ((_f = (_e = data.cals) === null || _e === void 0 ? void 0 : _e.extra) === null || _f === void 0 ? void 0 : _f.uin) || '',
            dependencies: data.dependencies || [],
            cals: data.cals || {},
            sub_cals: data.subAppCalsList || [],
        };
    }
}
exports.previewApp = previewApp;
